﻿using System;
using System.Collections.Generic;
using UniRx;
using static Magify.MagifyManager;

namespace Magify
{
    public class AnalyticsService
    {
        private readonly Subject<CustomEvent> _onBeforeSendEvent = new();
        private readonly Subject<CustomEvent> _onAfterSendEvent = new();

        public IObservable<CustomEvent> OnBeforeSendEvent => _onBeforeSendEvent;
        public IObservable<CustomEvent> OnAfterSendEvent => _onAfterSendEvent;

        public void TrackEvent(string name, Dictionary<string, object> @params)
        {
            var @event = new CustomEvent(name, @params);
            TrackEvent(@event);
        }

        public void TrackEvent(CustomEvent @event)
        {
            _onBeforeSendEvent.OnNext(@event);
            TrackCustomEvent(@event.Name, @event.Params);
            _onAfterSendEvent.OnNext(@event);
        }

        public void TweakAnalyticsConfig(int eventsGroupSize, int syncTimeInterval)
        {
            MagifyManager.TweakAnalyticsConfig(eventsGroupSize, syncTimeInterval);
        }

        public void ResetAnalyticsConfig()
        {
            MagifyManager.ResetAnalyticsConfig();
        }

        /// <summary>
        /// By design, it should only be used for debugging and testing.
        /// </summary>
        public void TweakUserLocale(string languageTag)
        {
            MagifyManager.TweakUserLocale(languageTag);
        }

        public void SetGameMode(string mode)
        {
            MagifyManager.GameState.SetGameMode(mode);
        }

        public void SetGameLevel(int level)
        {
            MagifyManager.GameState.SetGameLevel(level);
        }

        public void SetGameMaxLevel(int level)
        {
            MagifyManager.GameState.SetGameMaxLevel(level);
        }

        public void TrackIncomeTransaction(string source, List<BonusInfo> bonuses, ProductInfo product = null)
        {
            MagifyManager.TrackIncomeTransaction(source, bonuses, product);
        }

        public void TrackIncomeTransaction(string source, List<BonusInfo> bonuses, ProductObtainResult result)
        {
            TrackIncomeTransaction(source, bonuses, result.Id, result.PurchaseInfo);
        }

        public void TrackIncomeTransaction(string source, List<BonusInfo> bonuses, string productId = null, PurchaseInfo purchaseInfo = null)
        {
            var internalInfo = productId == null && purchaseInfo == null
                ? null
                : new ProductInfo
                {
                    ProductId = productId,
                    Price = purchaseInfo?.Price,
                    Currency = purchaseInfo?.Currency
                };

            MagifyManager.TrackIncomeTransaction(source, bonuses, internalInfo);
        }

        public void TrackExpenseTransaction(List<BonusInfo> bonuses)
        {
            MagifyManager.TrackExpenseTransaction(bonuses);
        }
    }
}