﻿using System.IO;
using JetBrains.Annotations;
using UnityEngine;

namespace Magify
{
    public partial class MagifyService
    {
        private const string PrefsName = "service.bin";

        public static MagifyService Instance { get; private set; }

        public static MagifyService Initialize([NotNull] MagifySettings settings, [CanBeNull] MagifyServiceArgs args)
        {
            if (Instance != null)
            {
                throw new MagifyAlreadyInitializedException();
            }
            // Prepare prefs
            var storagePath = Path.Combine(PackageInfo.PersistentPath, PrefsName);
            var prefs = ServicePrefs.Create(storagePath);
            var offerNativeEventsAccumulator = new LimitedTimeOfferNativeEventsAccumulator();
            MagifyManager.Initialize(new MagifyConfig
            {
#if !MAGIFY_3_OR_NEWER
                EditorMoq = args?.EditorMoq,
#endif
                PresenterSettings = settings,
                AppNameGP = settings.NameGP,
                AppNameIOS = settings.NameIOS,
                ConfigPathGP = settings.ConfigGP,
                ConfigPathIOS = settings.ConfigIOS,
                EditorDevice = args?.EditorConfigMode is RuntimePlatform.IPhonePlayer
                    ? EditorDevice.GetDefaultIOSDevice()
                    : EditorDevice.GetDefaultAndroidDevice(),
                Environment = args?.Environment ?? Magify.Environment.Staging,
                SubscriptionStatus = prefs.SubscriptionStatus.Value,
                IsGdprApplied = args?.IsGdprApplied,
                IsLoggingEnabled = args?.IsLoggingEnabled ?? true,
                RevenuePerCountryPath = settings.RevenueCountryPath,
                RevenueLevelsPath = settings.RevenueLevelsPath,
                DefaultCurrencyRatesPath = settings.CurrencyRatesPath,
                SubscriptionMultipliersPath = settings.TrialConversionPath,
            });
            offerNativeEventsAccumulator.StopEventsCatching();
            Instance = new MagifyService(settings, args, prefs);
            Instance.Offers.HandleOfferEventsInNextFrameStart(offerNativeEventsAccumulator).Forget();
#if !MAGIFY_3_OR_NEWER
            MagifyManager.TrackAppLaunch();
#endif
            return Instance;
        }

        public static void Terminate()
        {
            if (Instance == null) return;
            Instance.ShutDown();
            Instance = null;
        }
    }
}