using System;
using System.Collections.Generic;
using UnityEngine;

namespace Magify
{
    internal class CacheContent<TContent>
        where TContent : class
    {
        private readonly Action<TContent> _releaseCallback;
        private readonly Dictionary<string, int> _linksCounter = new();
        private readonly Dictionary<string, TContent> _caches = new();
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Content);

        public CacheContent(Action<TContent> releaseCallback = null)
        {
            _releaseCallback = releaseCallback;
        }

        public void Store(string url, TContent content)
        {
            _logger.Log($"Save content to cache\n{url}");
            _caches.Add(url, content);
            _linksCounter.Add(url, 0);
        }

        public TContent Get(string url)
        {
            _logger.Log($"Get content from cache\n{url}");
            _linksCounter[url]++;
            _logger.Log($"Get content from cache and increase reference to content became: {_linksCounter[url]}\n{url}");
            return _caches[url];
        }

        public bool Contains(string url)
        {
            return _caches.ContainsKey(url);
        }

        public void Release(string url)
        {
            var links = --_linksCounter[url];
            _logger.Log($"Decrease amount of reference to content, left {links}\n{url}");
            if (links == 0)
            {
                Remove(url);
            }
        }

        private void Remove(string url)
        {
            _logger.Log($"Remove content from cache\n{url}");
            var content = _caches[url];
            _caches.Remove(url);
            _linksCounter.Remove(url);
            _releaseCallback?.Invoke(content);
        }
    }
}