using System;
using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using Magify.Model;

namespace Magify
{
    internal class DefaultProductsContainer : IContextListener
    {
        private List<InAppProduct> _inAppProducts;
        private List<SubscriptionProduct> _subscriptionProducts;

        [CanBeNull]
        private List<DefaultProduct> _defaultProducts;

        public List<InAppProduct> InAppProducts
        {
            get
            {
                if (_defaultProducts == null)
                {
                    return new List<InAppProduct>(0);
                }
                _inAppProducts ??= _defaultProducts
                    .Where(c => c.Type is ProductType.Consumable or ProductType.NonConsumable)
                    .Select(c => new InAppProduct(c.ProductId, c.Type == ProductType.Consumable, null, c.ProductContext))
                    .ToList();
                return _inAppProducts;
            }
        }

        public List<SubscriptionProduct> SubscriptionProducts
        {
            get
            {
                if (_defaultProducts == null)
                {
                    return new List<SubscriptionProduct>(0);
                }
                _subscriptionProducts ??= _defaultProducts
                    .Where(c => c.Type is ProductType.Subscription)
                    .Select(c => new SubscriptionProduct(c.ProductId, null, c.ProductContext))
                    .ToList();
                return _subscriptionProducts;
            }
        }

        public void UpdateContext(CampaignsContext context, ContextKind kind)
        {
            switch (kind)
            {
                case ContextKind.Default when InAppProducts.Count == 0 && SubscriptionProducts.Count == 0:
                case ContextKind.Saved or ContextKind.Downloaded:
                    _inAppProducts = null;
                    _subscriptionProducts = null;
                    _defaultProducts = context.DefaultProducts;
                    break;
            }
        }

        public void ResetDefaultProducts()
        {
            _inAppProducts = null;
            _subscriptionProducts = null;
            _defaultProducts = null;
        }
    }
}