using System;
using JetBrains.Annotations;

namespace Magify
{
    internal interface IAuthorizationConfigBuilder
    {
        AuthorizationConfig AuthorizationConfig { get; }
    }

    internal class AuthorizationConfigBuilder : IInitializable, IAuthorizationConfigBuilder
    {
        private readonly string _applicationName;
        private readonly string _applicationVersion;
        private readonly PlatformAPI _platform;
        private readonly GeneralPrefs _prefs;

        private AuthorizationConfig _authorizationConfig;

        [NotNull]
        public AuthorizationConfig AuthorizationConfig
        {
            get
            {
                if (_authorizationConfig == null)
                {
                    throw new CannotGetAuthConfigBeforeInitException();
                }

                return _authorizationConfig;
            }
        }

        public AuthorizationConfigBuilder(string applicationName, string applicationVersion, PlatformAPI platform, GeneralPrefs prefs)
        {
            _applicationName = applicationName;
            _applicationVersion = applicationVersion;
            _platform = platform;
            _prefs = prefs;
        }

        void IInitializable.Initialize()
        {
            var locale = _platform.GetCurrentLocale();
            _authorizationConfig = new AuthorizationConfig
            {
                UserClientId = _prefs.ClientId,
                IsNewUser = _prefs.IsNewUser,
                DeviceName = _platform.DeviceName,
                DeviceVersion = _platform.DeviceVersion,

                ApplicationName = _applicationName,
                ApplicationVersion = _applicationVersion,
                SupportedOrientation = _platform.SupportedOrientation,

                SdkVersion = PackageInfo.Version,
                FirstLaunchDate = _prefs.FirstLaunchDate,

                LanguageCode = locale.LanguageWithRegion,
                CountryCode = locale.CountryCode,
            };
        }
    }
}