using System.Text;
using System.Threading;
using Magify.Model;
using Cysharp.Threading.Tasks;
using UnityEngine;
using UnityEngine.Networking;

namespace Magify
{
    internal class NetworkClient : INetworkClient
    {
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Network);

        public async UniTask<WebResponseMessage> SendAsync(string url, WebRequestMessage message, CancellationToken cancellationToken)
        {
            _logger.Log($"SendAsync with message => {message}");
            await TaskScheduler.SwitchToMainThread(cancellationToken);
            using var request = CreateRequest(url, message);
            await request.SendWebRequest().WithCancellation(cancellationToken);
            return request.ToWebRequestResponse(message);
        }

        private UnityWebRequest CreateRequest(string url, WebRequestMessage message)
        {
            var payload = new JsonRPCPayload(message.Method, message.Payload);
            var payloadString = JsonFacade.SerializeObject(payload);
            var payloadBytes = new UTF8Encoding().GetBytes(payloadString);

            var request = new UnityWebRequest(url, UnityWebRequest.kHttpVerbPOST)
            {
                uploadHandler = new UploadHandlerRaw(payloadBytes),
                downloadHandler = new DownloadHandlerBuffer(),
                timeout = 35,
            };
            request.SetRequestHeader("Content-Type", "application/json");
            if (message.AuthToken != null)
            {
                request.SetRequestHeader("Authorization", message.AuthToken);
            }

            return request;
        }
    }
}